"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

var url = _interopRequire(require("url"));

var http = _interopRequire(require("http"));

var assert = _interopRequire(require("assert"));

var dynalite = _interopRequire(require("dynalite"));

var async = _interopRequire(require("async"));

var aws = _interopRequire(require("aws-sdk"));

var levelup = _interopRequire(require("levelup"));

var concat = _interopRequire(require("concat-stream"));

var DynamoDOWN = _interopRequire(require("./dynamo-down"));

var server = undefined;
var dynamo = undefined;
var dynamoDown = undefined;
var db = undefined;

var httpStatusCodes = Object.keys(http.STATUS_CODES).map(function (key) {
  return { key: key, value: { message: http.STATUS_CODES[key] } };
});

var openDatabase = function (cb) {
  server = dynalite({ createTableMs: 0 });

  server.listen(function (err) {
    if (err) throw err;

    var _server$address = server.address();

    var address = _server$address.address;
    var port = _server$address.port;
    var endpoint = url.format({
      protocol: "http",
      hostname: address,
      port: port
    });

    dynamo = new aws.DynamoDB({
      endpoint: endpoint,
      region: "us-east-1",
      accessKeyId: "....................",
      secretAccessKey: "........................................"
    });

    cb();
  });
};

var closeDatabase = function (cb) {
  server.close(cb);
};

var createTable = function (cb) {
  var params = {
    TableName: "test",
    KeySchema: [{
      AttributeName: "type",
      KeyType: "HASH"
    }, {
      AttributeName: "key",
      KeyType: "RANGE"
    }],
    AttributeDefinitions: [{
      AttributeName: "type",
      AttributeType: "S"
    }, {
      AttributeName: "key",
      AttributeType: "S"
    }],
    ProvisionedThroughput: {
      ReadCapacityUnits: 5,
      WriteCapacityUnits: 5
    }
  };

  var oncreated = function (err) {
    if (err) throw err;

    dynamoDown = DynamoDOWN(dynamo);
    db = levelup("test/httpStatusCode", {
      db: dynamoDown,
      valueEncoding: "json"
    });

    setTimeout(cb, 1000);
  };

  dynamo.createTable(params, oncreated);
};

var deleteTable = function (cb) {
  var params = {
    TableName: "test"
  };

  var ondeleted = function (err) {
    if (err) throw err;

    setTimeout(cb, 1000);
  };

  dynamo.deleteTable(params, ondeleted);
};

var resetTable = function (cb) {
  async.series([deleteTable, createTable], cb);
};

var write = function (cb) {
  var ws = db.createWriteStream();

  for (var _iterator = httpStatusCodes[Symbol.iterator](), _step; !(_step = _iterator.next()).done;) {
    var code = _step.value;
    ws.write(code);
  }ws.end();
  setTimeout(cb, 1000);
};

var read = function (cb) {
  var rs = db.createReadStream({ reverse: true });
  var ws = concat(function (array) {
    assert.deepEqual(array.reverse(), httpStatusCodes);
    cb();
  });

  rs.pipe(ws);
};

var empty = function (cb) {
  dynamoDown.destroy("test/httpStatusCode", function (err) {
    if (err) return cb(err);

    var rs = db.createReadStream();
    var ws = concat(function (array) {
      assert.deepEqual(array, []);
      cb();
    });

    rs.pipe(ws);
  });
};

async.series([openDatabase, createTable, write, read, empty, deleteTable, closeDatabase]);